import click
import git
from gitutor.init.commands import init
from gitutor.save.commands import save


@click.group()
@click.pass_context
def cli(ctx):
    """ Git the easy way """
    #Check ctx was initialized
    ctx.ensure_object(dict)
    
    if ctx.invoked_subcommand != 'init':
        try:
            repo = git.Repo(".", search_parent_directories=True)
            ctx.obj['REPO'] = repo
            #print( f"Location {repo.working_tree_dir}" )
            #print(f"Remote from init: {repo.remote('origin').url} ")
        except Exception as e: 
            click.echo(e)
            #print("not git repo")
            exit()

cli.add_command(init)
cli.add_command(save)

def main():
    cli(obj={})

if __name__ == '__main__':
    cli(obj={})



import git
import os
import click
from github import Github
from github import GithubException
from git import GitCommandError

@click.command()
@click.pass_context
@click.option('-m', '--message', 'message')
@click.option('-p', '--push', is_flag=True)
def save(ctx, message, push):
    "Git add ., git commit -m, git pull, git push if possible"

    # Recover repo from context
    repo = ctx.obj['REPO']
    conflicted_files = get_conflict_files(repo)

    if not conflicted_files:

        #git add .
        if not message:
            message = click.prompt('Commit message')
    
        repo.git.add(A=True)

        #git commit -m ""
        try:
            repo.git.commit("-m", message)
        except GitCommandError:
            click.echo("No changes in local repository since last commit")

        if push:
            if repo.remotes.origin.exists():
                try:
                    repo.git.pull()
                except :
                    conflicted_files_merge = conflicts_from_merge(repo)

                    if conflicted_files_merge:
                        click.echo('Merge conflicts in:')
                        for conflict_file in conflicted_files_merge: 
                            click.echo("- " + conflict_file)
                        click.echo('Please fix conflicts then use "gt save -p" again')    
                else:
                    repo.git.push()

            else:
                click.echo('Remote repository does not exist!')
    else:
        click.echo('Please fix the following conflicts then use "gt save -p" again')
        for conflicted_file in conflicted_files:
            click.echo(conflicted_file)

def conflicts_from_merge(repo):
    unmerged_blobs = repo.index.unmerged_blobs()
    error_array = []
    # We're really interested in the stage each blob is associated with.
    # So we'll iterate through all of the paths and the entries in each value
    # list, but we won't do anything with most of the values.
    for path in unmerged_blobs:
        list_of_blobs = unmerged_blobs[path]
        for (stage, blob) in list_of_blobs:
            # Now we can check each stage to see whether there were any conflicts
            if stage != 0:
                if path not in error_array:
                    error_array.append(path)
    return error_array

def get_conflict_files(repo):
    matched_lines=[]
    try:
        repo.git.diff("--check")
    except GitCommandError as e:
        matched_lines = [line for line in e.stdout.split("'")[1].split('\n') if "conflict" in line]
    return matched_lines


import click
from github import Github
from github import GithubException
import git
import os


@click.command()
@click.pass_context
@click.option('-u', '--user', 'user_name' )
@click.option('-p', '--password', 'password', hide_input=True)
@click.option('-l', '--local', is_flag=True)
@click.option('-r', '--repo-name')
def init(ctx, user_name, password,local, repo_name):
    "Create git repo and github remote"
    if is_initialized_repo():
        click.echo('This is already a git repo!')
        return

    if local:
        init_local_repo()
        return
    if not user_name:
        user_name = click.prompt('Username')
    
    if not password:
        password = click.prompt('Password', hide_input=True)

    if not repo_name:
        repo_name = click.prompt('Repo name')
    
    try:
        g = Github(user_name, password)
        user = g.get_user()
        click.echo('Creating github repo...')
        remote_repo = user.create_repo(repo_name)
    except GithubException as e:
        error_code = e.args[0]
        if error_code == 401 :
            click.echo('Problem with credentials!')
        else:
            if error_code == 422 :
                click.echo('Repo name already used!')
    else:
        local_repo = init_local_repo()
        local_repo.create_remote('origin', remote_repo.clone_url)
        click.echo('Push to origin...')
        local_repo.git.push('-u', 'origin', 'master')
        click.echo('Ready!')

def init_local_repo():
    repo_dir = os.getcwd()
    file_name = os.path.join(repo_dir, 'README.md')

    repo = git.Repo.init(repo_dir)
    #Create empty README.md
    open(file_name, 'wb').close()
    repo.index.add([file_name])
    repo.index.commit('first commit')
    return repo

def is_initialized_repo():
    try:
        git.Repo('.', search_parent_directories=True)
    except:
        return False
    else:
        return True







# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gitutor', 'gitutor.init', 'gitutor.save']

package_data = \
{'': ['*']}

install_requires = \
['GitPython==3.1.3',
 'click==7.1.2',
 'gitdb==4.0.5',
 'pygithub==1.51',
 'pyinquirer==1.0.3',
 'smmap==3.0.4']

entry_points = \
{'console_scripts': ['gt = gitutor.cli:main']}

setup_kwargs = {
    'name': 'gitutor',
    'version': '0.2.2',
    'description': 'Making git easier!',
    'long_description': None,
    'author': 'AMIA',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
